// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.services' is found in services.js
// 'starter.controllers' is found in controllers.js
angular.module('starter', ['ionic', 'starter.controllers', 'starter.services', 'ngCordova'])

  .constant('xc', {
    app: {
      id: 'teacher',
      advTime: 3,     //广告显示时间，单位：秒
      pageSize: 5,   //单页显示条数
      cache: true,    //是否启用缓存
      timeout: 60000,   //5秒钟超时
      interval: 30000, //0.5分钟轮询一次
      subjectColors: {语文: 'positive', 数学: 'balanced', 英语: 'energized', 物理: 'assertive', 化学: 'royal', 政治: 'calm', 历史: 'dark'}
    },
    server: {
     // domain: 'http://www.cfec.com/',
     // base: 'http://www.cfec.com/app/api/',
    // base: 'http://192.168.1.50:8800/app/api/',
      domain: 'http://www.41lan.com/',
      base: 'http://www.41lan.com/app/api/',
      cdn: 'http://img.41lan.com/'
    }
  })

  .run(function ($ionicPlatform, $location, $rootScope, $cordovaAppVersion, $cordovaToast, $cordovaNetwork, $cordovaKeyboard, $ionicHistory, ImService, UpdateService) {

    $rootScope.network = {isOnline: true};

    //物理返回按钮控制&双击退出应用
    $ionicPlatform.registerBackButtonAction(function (e) {
      //判断处于哪个页面时双击退出
      var path = $location.path();

      if (path == '/tab/home' || path == '/tab/teach' || path == '/tab/class' || path == '/tab/font' || path == '/tab/account' || path == '/login') {
        if ($rootScope.backButtonPressedOnceToExit) {
          ionic.Platform.exitApp();
        } else {
          $rootScope.backButtonPressedOnceToExit = true;
          $cordovaToast.showShortBottom('再按一次退出系统');
          setTimeout(function () {
            $rootScope.backButtonPressedOnceToExit = false;
          }, 2000);
        }
      } else if ($ionicHistory.backView()) {
        if ($cordovaKeyboard.isVisible()) {
          $cordovaKeyboard.close();
        } else {
          $ionicHistory.goBack();
        }
      }
      else {
        $rootScope.backButtonPressedOnceToExit = true;
        $cordovaToast.showShortBottom('再按一次退出系统');
        setTimeout(function () {
          $rootScope.backButtonPressedOnceToExit = false;
        }, 2000);
      }
      e.preventDefault();
      return false;
    }, 101);

    $ionicPlatform.ready(function () {

      // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
      // for form inputs)
      if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
        cordova.plugins.Keyboard.disableScroll(true);
      }
      if (window.StatusBar) {
        StatusBar.styleDefault();
      }

      //ios版本注释掉更新代码
      if (ionic.Platform.isAndroid())
        UpdateService.checkUpdate();

      ImService.startServer();

      $cordovaAppVersion.getVersionNumber().then(function (v) {
        $rootScope.v = v;
      });
    });

    $ionicPlatform.on('offline', function () {
      ImService.stopServer();
      $rootScope.network.isOnline = false;
      $cordovaToast.showShortBottom('网络已经断开');
    });

    $ionicPlatform.on('online', function () {
      ImService.startServer();
      $rootScope.network.isOnline = true;
      $cordovaToast.showShortBottom('网络已经连接');
    });

    $ionicPlatform.on('pause', function () {
      console.log('pause');
      ImService.stopServer();
    });

    $ionicPlatform.on('resume', function () {
      console.log('resume');
      ImService.startServer();
    });

    $ionicPlatform.on('volumedownbutton', function () {
      console.log('volumedownbutton');
    });

    $ionicPlatform.on('volumeupbutton', function () {
      console.log('volumeupbutton');
    });

    $ionicPlatform.on('batterylow', function () {
      console.log('batterylow');
    });
  })

  .config(function ($stateProvider, $httpProvider, $ionicConfigProvider, $urlRouterProvider, ionGalleryConfigProvider) {

    // Use x-www-form-urlencoded Content-Type
    $httpProvider.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded;charset=utf-8';

    /**
     * The workhorse; converts an object to x-www-form-urlencoded serialization.
     * @param {Object} obj
     * @return {String}
     */
    var param = function (obj) {
      var query = '', name, value, fullSubName, subName, subValue, innerObj, i;

      for (name in obj) {
        value = obj[name];

        if (value instanceof Array) {
          for (i = 0; i < value.length; ++i) {
            subValue = value[i];
            fullSubName = name + '[' + i + ']';
            innerObj = {};
            innerObj[fullSubName] = subValue;
            query += param(innerObj) + '&';
          }
        }
        else if (value instanceof Object) {
          for (subName in value) {
            subValue = value[subName];
            fullSubName = name + '[' + subName + ']';
            innerObj = {};
            innerObj[fullSubName] = subValue;
            query += param(innerObj) + '&';
          }
        }
        else if (value !== undefined && value !== null)
          query += encodeURIComponent(name) + '=' + encodeURIComponent(value) + '&';
      }

      return query.length ? query.substr(0, query.length - 1) : query;
    };

    // Override $http service's default transformRequest
    $httpProvider.defaults.transformRequest = [function (data) {
      return angular.isObject(data) && String(data) !== '[object File]' ? param(data) : data;
    }];

    $ionicConfigProvider.backButton.text('返回');
    $ionicConfigProvider.platform.ios.tabs.style('standard');
    $ionicConfigProvider.platform.ios.tabs.position('bottom');
    $ionicConfigProvider.platform.android.tabs.style('standard');
    $ionicConfigProvider.platform.android.tabs.position('bottom');

    $ionicConfigProvider.platform.ios.navBar.alignTitle('center');
    $ionicConfigProvider.platform.android.navBar.alignTitle('center');
    $ionicConfigProvider.platform.ios.backButton.previousTitleText('').icon('ion-ios-arrow-left');
    $ionicConfigProvider.platform.android.backButton.previousTitleText('').icon('ion-ios-arrow-left');

    $ionicConfigProvider.platform.ios.views.transition('ios');
    $ionicConfigProvider.platform.android.views.transition('android');

    $ionicConfigProvider.views.transition('platform');
    $ionicConfigProvider.views.maxCache(0);

    ionGalleryConfigProvider.setGalleryConfig({
      action_label: '关闭', //预览关闭按钮文字
      template_slider: 'templates/slider.html',
      toggle: false,          //
      row_size: 3
    });

    // Ionic uses AngularUI Router which uses the concept of states
    // Learn more here: https://github.com/angular-ui/ui-router
    // Set up the various states which the app can be in.
    // Each state's controller can be found in controllers.js
    $stateProvider

    // setup an abstract state for the tabs directive

      .state('login', {
        url: '/login',
        templateUrl: 'templates/login.html'
      })

      .state('welcome', {
        url: '/welcome',
        templateUrl: 'templates/welcome.html'
      })

      .state('tab', {
        url: '/tab',
        abstract: true,
        templateUrl: 'templates/tabs.html'
      })

      //教学管理
      .state('tab.teach', {
        url: '/teach',
        views: {
          'tab-teach': {
            templateUrl: 'templates/tab-teach.html',
            controller: 'TeachCtrl'
          }
        }
      })

      //教学资讯阅读
      .state('tab.teach-article', {
        url: '/teach/article/:artId',
        views: {
          'tab-teach': {
            templateUrl: 'templates/article.html',
            controller: 'ArticleCtrl'
          }
        }
      })

      //首页
      .state('tab.home', {
        url: '/home',
        views: {
          'tab-home': {
            templateUrl: 'templates/tab-home.html',
            controller: 'HomeCtrl'
          }
        }
      })

      //课程列表
      .state('tab.course', {
        url: '/course',
        views: {
          'tab-home': {
            templateUrl: 'templates/tab-course.html',
            controller: 'CourseListCtrl'
          }
        }
      })

      //首页资讯阅读
      .state('tab.home-article', {
        url: '/home/article/:artId',
        views: {
          'tab-home': {
            templateUrl: 'templates/article.html',
            controller: 'ArticleCtrl'
          }
        }
      })

      //专栏起始页
      .state('tab.special', {
        url: '/special',
        views: {
          'tab-home': {
            templateUrl: 'templates/special.html',
            controller: 'SpecialCtrl'
          }
        }
      })

      //专栏列表页
      .state('tab.special-list', {
        url: '/special/list/:cateId/:name',
        views: {
          'tab-home': {
            templateUrl: 'templates/article-list.html',
            controller: 'SpecialListCtrl'
          }
        }
      })

      //精华
      .state('tab.essence', {
        url: '/essence',
        views: {
          'tab-home': {
            templateUrl: 'templates/article-list.html',
            controller: 'EssenceCtrl'
          }
        }
      })

      //region  tab-class

      //班级管理
      .state('tab.class', {
        url: '/class',
        views: {
          'tab-class': {
            templateUrl: 'templates/tab-class.html'
          }
        }
      })

      //班级列表
      .state('tab.class-list', {
        url: '/class/list',
        views: {
          'tab-class': {
            templateUrl: 'templates/class-list.html',
            controller: 'ClassListCtrl'
          }
        }
      })

      //学生列表
      .state('tab.student-list', {
        url: '/student/list/:id/:name',
        views: {
          'tab-class': {
            templateUrl: 'templates/student-list.html',
            controller: 'StudentListCtrl'
          }
        }
      })

      //学生详细信息
      .state('tab.student-view', {
        url: '/student/view/:studentId',
        views: {
          'tab-class': {
            templateUrl: 'templates/student.html',
            controller: 'StudentCtrl'
          }
        }
      })

      //通知列表
      .state('tab.notice-list', {
        url: '/notice/list',
        views: {
          'tab-class': {
            templateUrl: 'templates/notice-list.html',
            controller: 'NoticeListCtrl'
          }
        }
      })

      //通知详细
      .state('tab.notice-view', {
        url: '/notice/view/:id',
        views: {
          'tab-class': {
            templateUrl: 'templates/notice-view.html',
            controller: 'NoticeViewCtrl'
          }
        }
      })

      //家庭作业列表
      .state('tab.homework-list', {
        url: '/homework/list',
        views: {
          'tab-class': {
            templateUrl: 'templates/homework-list.html',
            controller: 'HomeworkListCtrl'
          }
        }
      })

      // 班级相册
      .state('tab.gallery-index', {
        url: '/gallery/index',
        views: {
          'tab-class': {
            templateUrl: 'templates/gallery-index.html',
            controller: 'GalleryIndexCtrl'
          }
        }
      })

      //班级相册
      .state('tab.gallery-list', {
        url: '/gallery/list/:classId/:name',
        views: {
          'tab-class': {
            templateUrl: 'templates/gallery.html',
            controller: 'GalleryCtrl'
          }
        }
      })

      //聊天列表
      .state('tab.chat-list', {
        url: '/chat/list',
        cache: false,
        views: {
          'tab-class': {
            templateUrl: 'templates/chat-list.html',
            controller: 'ChatListCtrl'
          }
        }
      })

      //聊天窗口
      .state('tab.chat', {
        url: '/chat/:receiver/:groupId/:name',
        cache: false,
        views: {
          'tab-class': {
            templateUrl: 'templates/chat.html',
            controller: 'ChatCtrl'
          }
        }
      })

      //显示家长信息
      .state('tab.user', {
        url: '/user/:userId',
        views: {
          'tab-class': {
            templateUrl: 'templates/user.html',
            controller: 'UserCtrl'
          }
        }
      })

      //下载用户列表
      .state('tab.download-list', {
        url: '/download/list',
        views: {
          'tab-class': {
            templateUrl: 'templates/download-list.html',
            controller: 'DownloadListCtrl'
          }
        }
      })

      //下载用户列表明细
      .state('tab.download-view', {
        url: '/download/view/:type/:classId/:className',
        views: {
          'tab-class': {
            templateUrl: 'templates/download-view.html',
            controller: 'DownloadViewCtrl'
          }
        }
      })

      //   endregion

      // region contact

      .state('tab.contact-list', {
        url: '/contact/list',
        views: {
          'tab-contact': {
            templateUrl: 'templates/contact-list.html',
            controller: 'ContactListCtrl'
          }
        }
      })

      // endregion

      //region tab-account

      //个人中心
      .state('tab.account', {
        url: '/account',
        cache: false,
        views: {
          'tab-account': {
            templateUrl: 'templates/tab-account.html',
            controller: 'AccountCtrl'
          }
        }
      })

      //字体设置
      .state('tab.font', {
        url: '/font',
        views: {
          'tab-account': {
            templateUrl: 'templates/font.html',
            controller: 'FontCtrl'
          }
        }
      })

      //关于我们
      .state('tab.about-us', {
        url: '/about/us',
        views: {
          'tab-account': {
            templateUrl: 'templates/about-us.html'
          }
        }
      })

      //意见反馈
      .state('tab.account-advice', {
        url: '/account/advice',
        views: {
          'tab-account': {
            templateUrl: 'templates/advice.html',
            controller: 'AdviceCtrl'
          }
        }
      })

      //endregion

      //个人资料
      .state('tab.profile', {
        url: '/profile',
        cache: false,
        views: {
          'tab-account': {
            templateUrl: 'templates/tab-profile.html',
            controller: 'profileCtrl'
          }
        }
      });

    if (!window.localStorage.getItem('user')) {
      $urlRouterProvider.otherwise('login');
    } else {
      $urlRouterProvider.otherwise('tab/home');
    }
    //$urlRouterProvider.otherwise('/welcome');
  });
