angular.module('starter.services', [])

//统一请求入口
  .factory('http', function ($http, $ionicLoading, $rootScope, $cordovaToast, $timeout, $location, xc) {

    window.authorize = true;

    //成功回调
    function successCallback(response) {
      console.info('request success: ' + JSON.stringify(response));
      return response;
    }

    //失败回调
    function errorCallback(response, error) {

      console.error('request error: ' + JSON.stringify(response));

      if (typeof error === 'function')
        error(response);
      else {
        $ionicLoading.show({
          template: '<div><i class="icon icon-2x ion-sad-outline"></i></div>数据请求失败...',
          noBackdrop: true,
          hideOnStateChange: true,
          duration: 3000
        });
      }

      return response;
    }

    //授权失败
    function authorizeFalse(msg, error) {

      window.authorize = false;

      if (typeof error === 'function')
        error();
      else {
        $ionicLoading.show({
          template: msg || '用户登录超时,请重新登录',
          duration: 3000
        });
        $timeout(function () {
          window.authorize = true;
          $ionicLoading.hide();
          $location.url('/login');
        }, 3000);
      }
    }

    //请求方法
    function request(options) {

      var url = options.url;

      if (options.url.indexOf('http') == -1) {
        url = xc.server.base + options.url;
      }

      var params = options.params || {};

      if ($location.$$url.indexOf('tab') > -1) {

        var token = null;

        try {
          var user = window.localStorage.getItem(xc.app.userCache);

          if (user) {
            user = JSON.parse(user);
            token = user.accessToken;
          }
        } catch (e) {
        }

        if (token)
          params.accessToken = token;
      }

      console.log('request url: ' + url);

      console.log('request par: ' + JSON.stringify(params));

      $ionicLoading.show({
        template: '<ion-spinner icon="lines" class="spinner-assertive"></ion-spinner><br>加载中...',
        noBackdrop: true,
        duration: xc.app.laodTimeout,
        delay: 500
      });

      var postData = null;

      $http(
        {
          method: options.method.toUpperCase(),
          url: url,
          params: params,
          data: options.data,
          timeout: xc.app.laodTimeout
        }).then(function (response) {

        $ionicLoading.hide();

        var res = response.data;

        console.log('通讯成功');

        //交互成功
        if (res.code == 0) {
          if (typeof options.success === 'function') {
            options.success(res);
          }
          else
            successCallback(res);
        }
        else if (res == 801 || res == 802) {
          authorizeFalse();
        }
        //系统异常
        else {
          errorCallback(response, options.error);
        }

      }, function (response) {
        console.log('通讯失败');
        $ionicLoading.hide();
        errorCallback(response, options.error);
      });
    }

    function action(options) {

      if (window.authorize) {

        //检查网络
        //如果网络可用
        if ($rootScope.network.isOnline) {
          return request(options);
        } else {
          $cordovaToast.showShortBottom('当前没有可用的网络连接');
        }
      }
    }

    return {
      get: function (url, params, success, error) {
        action({
          method: 'GET',
          url: url,
          params: params,
          success: success,
          error: error
        });
      },
      post: function (url, params, data, success, error) {
        action({
          method: 'POST',
          url: url,
          data: data,
          params: params,
          success: success,
          error: error
        });
      }
    }
  })

  //缓存类
  .factory('CacheService', function () {

    var _size = {value: 0, unit: 'KB'};

    var resize = function () {

      var length = 0;

      for (var i = 0, len = window.localStorage.length; i < len; i++) {
        var key = window.localStorage.key(i);
        if (key == 'user') continue;
        var data = window.localStorage.getItem(key);
        if (data) length += data.length;
      }

      var units = ['B', 'KB', 'M', 'G'];

      var st = 1024;

      var unitIndex = 0;

      var size = (length * 4 );

      while (size > st && unitIndex < 4) {
        size %= st;
        unitIndex++;
      }

      if (size > 0) {
        _size.value = size.toFixed(1);
        _size.unit = units[unitIndex];
      }
      else {
        _size.value = 0;
        _size.unit = 'KB';
      }
    };

    return {
      size: _size,
      set: function (key, value) {
        var val = value;
        if (typeof value == 'object')
          val = JSON.stringify(value);
        window.localStorage.setItem(key, val);
        resize();
      },
      get: function (key) {
        var value = window.localStorage.getItem(key);
        try {
          return JSON.parse(value);
        } catch (e) {
          return value;
        }
      },
      remove: function (key) {
        window.localStorage.removeItem(key);
        resize();
      },
      clear: function () {
        var user = window.localStorage.getItem("user");
        window.localStorage.clear();
        if (user)
          window.localStorage.setItem("user", user);
        _size.value = 0;
        _size.unit = 'KB';
      }
    };
  })

  //用户类
  .factory('UserService', function ($http, $state, $ionicModal, $ionicLoading, $ionicPopup, $interval, xc, CacheService) {

    var currentUser = CacheService.get('user') || {isLogin: false, username: '', password: ''};

    //60秒以后重新启动获取验证码按钮
    function timetick($scope) {

      $scope.second = 60;
      $scope.phonecode = {text: '重新发送验证码', isvalid: false};

      changeText();

      $scope.timer = $interval(function () {
        changeText();
      }, 1000);

      function changeText() {
        if ($scope.second == 0) {
          $scope.phonecode.isvalid = true;
          $scope.phonecode.text = '重新发送验证码';
          $interval.cancel($scope.timer);
          delete $scope.timer;
          return;

        }
        $scope.phonecode.text = $scope.second + '秒后 重新发送验证码';
        $scope.second--;
      }
    }

    //Toast
    function toast(text) {
      $ionicLoading.show({
        template: text,
        animation: 'fade-in',
        showBackdrop: false,
        duration: 2000,
        showDelay: 0
      });
    }

    return {
      user: currentUser,
      name: currentUser.name,
      id: currentUser.id,
      updateAvatar: function (avatar) {
        var _user = CacheService.get('user');
        currentUser.avatar = xc.server.cdn + '/' + avatar;
        _user.avatar = xc.server.cdn + '/' + avatar;
        CacheService.set('user', _user);
      },
      login: function () {
        $ionicLoading.show({
          template: '正在登陆...',
          animation: 'fade-in',
          showBackdrop: true,
          showDelay: 0
        });
        var deviceInformation = ionic.Platform.device();

        $http.get(xc.server.base + 'admin/login', {
          params: {
            uuid: deviceInformation.uuid,
            appid: 'teacher',
            username: currentUser.username,
            password: currentUser.password
          }
        })
          .error(function () {
            console.error('从服务器获取数据失败!');
            currentUser.isLogin = false;
            $ionicLoading.hide();
            $ionicPopup.alert({
              title: '登陆失败',
              template: '与服务器通讯失败，请检查网络！',
              okText: '确认'
            });

          })
          .then(function (response) {

            var data = response.data.data;

            if (response.data.code === 0) {

              currentUser.isLogin = true;
              currentUser.password = null;

              for (var key in data) {
                currentUser[key] = data[key];
              }
              CacheService.set('user', currentUser);
              //跳
              $state.go('tab.home');
            } else {
              $ionicPopup.alert({
                title: '登陆失败',
                template: response.data.msg || '与服务器通讯失败，请检查网络！',
                okText: '确认'
              });
            }
            $ionicLoading.hide();
          });
      },
      logout: function () {
        var confirmPopup = $ionicPopup.confirm({
          title: '退出确认',
          template: '您确认要退出登陆吗？',
          cancelText: '取消',
          okText: '确认'
        });
        confirmPopup.then(function (res) {
          if (res) {
            CacheService.remove('user');
            currentUser.isLogin = false;
            currentUser.username = null;
            currentUser.password = null;
            //跳
            $state.go('login');
          }
        });
      },
      findpwd: function ($scope) {
        $scope.user.telphone = '';
        $ionicModal.fromTemplateUrl('templates/telphone.html', {
          scope: $scope
        }).then(function (modal) {
            $scope.telModel = modal;
            $scope.telModel.show();
          }
        );
      },
      getuserInfo: function ($scope) {
        $scope.user.account = null;
        $scope.accounts = null;

        $ionicLoading.show({
          template: '数据加载中...',
          animation: 'fade-in',
          showBackdrop: false,
          duration: 5000,
          showDelay: 0
        });

        $ionicModal.fromTemplateUrl('templates/account.html', {
          scope: $scope
        }).then(function (modal) {

            $scope.button.sendCodeText = '发送验证码';

            $scope.accountModel = modal;
            $scope.accountModel.show();

            http.get(xc.server.base + 'admin/getuser', {
              params: {
                telphone: $scope.user.telphone,
                role: 'teacher'
              }
            }).error(function () {
              $scope.error.accountMessage = '从服务器获取数据失败!';
            }).then(function (response) {
              var value = response.data;
              if (value.code === 0) {
                $scope.accounts = [];
                $scope.error.accountMessage = value.msg;
              } else {
                $scope.error.accountMessage = null;
                $scope.accounts = value.data;
                if ($scope.accounts.length > 0) {
                  $scope.user.account = value.data[0].account;
                }
              }

              $ionicLoading.hide();
            });
          }
        );
      },
      closeTelModel: function ($scope) {
        $scope.telModel.remove();
      },
      closeAccountModel: function ($scope) {
        $scope.accountModel.remove();
      },
      closeResetPwdModel: function ($scope) {
        $scope.resetPwdModel.remove();
        if ($scope.timer != null) {
          $interval.cancel($scope.timer);
        }
        delete $scope.timer;
      },
      sendcode: function ($scope) {
        var that = this;

        $scope.button.sendCodeText = '发送中…';
        console.info($scope.user);

        //获取验证码
        http.get('admin/getcode', {
          telphone: $scope.user.telphone
        }, function (response) {
          var value = response.data;

          $scope.button.sendCodeText = '发送验证码';

          switch (value.code) {
            case 0:
            case 2:
              $scope.error.accountMessage = value.msg;
              break;
            case 1:
              console.info(value);
              $scope.error.accountMessage = null;

              $ionicLoading.show({
                template: '验证码发送成功',
                animation: 'fade-in',
                showBackdrop: false,
                duration: 2000,
                showDelay: 0
              });

              //弹出重置密码
              that.resetPwd($scope);

              break;
          }
        }, function () {
          $scope.error.accountMessage = '从服务器获取数据失败!';
          $scope.button.sendCodeText = '发送验证码';
        });
      },
      resetPwd: function ($scope) {
        $ionicModal.fromTemplateUrl('templates/resetpwd.html', {
          scope: $scope
        }).then(function (modal) {

            $scope.resetPwdModel = modal;
            $scope.resetPwdModel.show();

            //重新启动获取验证码按钮
            timetick($scope);
          }
        );
      },
      resendCode: function ($scope) {
        var that = this;
        $scope.phonecode = {isvalid: false};
        http.get('admin/getcode', {
          telphone: $scope.user.telphone
        }, function (value) {
          timetick($scope);
        });
      },
      postPassword: function ($scope) {

        var that = this;

        $http.get(xc.server.base + 'admin/updatePwd', {
          params: {
            account: $scope.user.account,
            telphone: $scope.user.telphone,
            code: $scope.user.code,
            password: $scope.user.password
          }
        }).error(function () {
          $scope.error.accountMessage = '从服务器获取数据失败!';
        }).then(function (response) {
            var value = response.data;
            if (value.code == 0) {
              $scope.error.accountMessage = value.msg;
            } else {
              //密码修改成功
              $scope.user.username = $scope.user.account;

              toast('密码修改成功');

              $interval.cancel($scope.timer);
              $scope.timer = null;

              $scope.user.code = '';
              $scope.user.password2 = '';

              that.closeAccountModel($scope);
              that.closeResetPwdModel($scope);
              that.closeTelModel($scope);
            }
          }
        );
      },
      updatePassword: function ($scope) {

        $scope.message = '';
        $scope.user.oldpassword = '';
        $scope.user.newpassword = '';

        var that = this;
        var popup = $ionicPopup.show({
          title: '修改密码',
          scope: $scope,
          template: '<div class="list">' +
          '<label class="item item-input"><input type="password" placeholder="原始密码" ng-model="user.oldpassword"></label>' +
          '<label class="item item-input"><input type="password" placeholder="新密码(不少于6位)" ng-model="user.newpassword"></label>' +
          '<div class="text-left assertive" ng-bind="message" style="margin-top: 5px;"></div>' +
          '</div>',
          buttons: [
            {
              text: '取消',
              type: 'button button-light'
            }, {
              text: '确定',
              type: 'button-positive',
              onTap: function (e) {

                if ($scope.user.oldpassword.length == 0) {
                  $scope.message = '原始密码不能为空';
                  e.preventDefault();
                  return false;
                }

                if ($scope.user.newpassword.length < 6) {
                  $scope.message = '新密码不能少于6位';
                  e.preventDefault();
                  return false;
                }

                $http.get(xc.server.base + 'admin/resetPwd', {
                  params: {
                    aid: '' + that.user.id,
                    oldpwd: $scope.user.oldpassword,
                    password: $scope.user.newpassword
                  }
                }).error(function () {
                  toast('从服务器获取数据失败!');
                }).then(function (response) {
                    var value = response.data;
                    if (value.code == 0) {
                      $scope.message = value.msg;
                    } else {
                      $scope.user.oldpassword = '';
                      $scope.user.newpassword = '';
                      popup.close();
                      toast('密码修改成功');
                    }
                  }
                );
                e.preventDefault();
              }
            }
          ]
        });
      },
      updateRealName: function ($scope) {
        var that = this;
        $ionicPopup.prompt({
          scope: $scope,
          title: '姓名设置',
          template: '<input type="text" ng-model="profile.name" placeholder="姓名" style="padding-left: 10px;" />',
          okText: '确定',
          cancelText: '取消'
        }).then(function (res) {
          if (typeof res != 'undefined') {
            $http.get(xc.server.base + 'admin/updateRealName', {
              params: {
                aid: '' + that.user.id,
                realName: decodeURIComponent(decodeURIComponent($scope.profile.name))
              }
            }).error(function () {
              toast('从服务器获取数据失败!');
            }).then(function (response) {

                var value = response.data;
                toast(value.msg);

                if (value.code == 1) {
                  var user = $scope.profile;
                  $scope.user.realname = user.name;
                  CacheService.remove('user');
                  CacheService.set('user', user);
                }
              }
            );
          }
        });
      },
      updateTelPhone: function ($scope) {
        var that = this;
        $ionicPopup.prompt({
          scope: $scope,
          title: '手机号设置',
          template: '<input type="text" ng-model="profile.telphone" placeholder="手机号" style="padding-left: 10px;" />',
          okText: '确定',
          cancelText: '取消'
        }).then(function (res) {
          if (typeof res != 'undefined') {
            $http.get(xc.server.base + 'admin/updateTelphone', {
              params: {
                aid: '' + that.user.id,
                telphone: $scope.profile.telphone
              }
            }).error(function () {
              toast('服务器获取数据失败!');
            }).then(function (response) {
                var value = response.data;
                toast(value.msg);

                if (value.code == 1) {

                  //将姓名写回到缓存中
                  var user = $scope.profile;
                  $scope.user.telphone = user.telphone;
                  CacheService.remove('user');
                  CacheService.set('user', user);
                }
              }
            );
          }
        });
      }
    };
  })

  //升级程序
  .factory('UpdateService', function ($rootScope, $cordovaAppVersion, $cordovaFileTransfer, $timeout, $ionicLoading, $cordovaFileOpener2, $http, $ionicPopup, xc) {

    //检查更新
    var checkUpdate = function () {
      $cordovaAppVersion.getVersionNumber().then(function (v) {
        var url = xc.server.domain + 'update/checkNew/' + xc.app.id + '/android/' + v;
        $http.get(url)
          .error(function () {
            console.log('从服务器获取数据失败!');
          })
          .then(function (response) {
              var data = response.data;
              var compatible_binary = data.compatible_binary;
              var update_available = data.update_available;
              if (compatible_binary && update_available) {
                var url = xc.server.domain + data.update.url;
                showUpdateConfirm(url, data.info || '');
              }
            }
          );
      });
    };

    /**
     *
     * @param url
     * @param info
     */
    var showUpdateConfirm = function (url, info) {

      var confirmPopup = $ionicPopup.confirm({
        title: '版本升级',
        template: info.toString().replace(/\n/g, '<br />'),
        cancelText: '取消',
        okText: '升级'
      });

      confirmPopup.then(function (res) {
        if (res) {
          $rootScope.process = 0;
          $ionicLoading.show({
            template: '<ion-spinner icon="bubbles" class="spinner-assertive spinner spinner-bubbles"></ion-spinner><br>已经下载：{{process}}%'
          });
          var targetPath = cordova.file.externalDataDirectory + "update.apk";//APP下载存放的路径，可以使用cordova file插件进行相关配置
          var trustHosts = true;
          var options = {};
          $cordovaFileTransfer.download(url, targetPath, options, trustHosts).then(function (result) {
            // 打开下载下来的APP

            //console.log(JSON.stringify(result));

            $cordovaFileOpener2.open(targetPath, 'application/vnd.android.package-archive'
            ).then(function () {

            }, function (err) {

            });
            $ionicLoading.hide();
          }, function (err) {
            alert('下载失败');
          }, function (progress) {
            //进度，这里使用文字显示下载百分比
            $timeout(function () {
              var downloadProgress = (progress.loaded / progress.total) * 100;
              $rootScope.process = Math.floor(downloadProgress);
              if (downloadProgress > 99) {
                $ionicLoading.hide();
              }
            })
          });

        } else {
          // 取消更新
        }
      });
    };

    return {
      checkUpdate: checkUpdate
    }
  })

  //资讯
  .factory('ArticleService', function (http, $ionicModal, $ionicPopup, $ionicLoading, xc, CacheService) {

    return {
      request: function (options) {
        options.params.pageNum = options.params.pageNo;
        http.get('getArticeList', options.params, function (response) {

          var data = response.data;

          options.params.isHasNext = data.isHasNextPage;

          options.params.isLastPage = data.isLastPage;

          options.params.pageNo = data.pageNum;

          if (options.params.pageNo == 1) {
            options.list = [];
          }

          for (var i = 0, len = data.list.length; i < len; i++) {
            var d = data.list[i];
            d.summary = d.summary.trim();
            options.list.push(d);
          }

          CacheService.set(options.key, options);

          if (typeof options.always == 'function') {
            options.always(response);
          }
        });
      }
    };
  })

  //消息列表
  .factory('ImGroupService', function (http) {

    return {
      request: function (options) {
        options.params.pageNum = options.params.pageNo;
        http.get('im/group', options.params, function (response) {

          var data = response.data;

          options.params.isHasNext = data.isHasNextPage;

          options.params.isLastPage = data.isLastPage;

          options.params.pageNo = data.pageNum;

          if (options.params.pageNo == 1) {
            options.list = [];
          }

          for (var i = 0, len = data.list.length; i < len; i++) {
            var d = data.list[i];
            options.list.push(d);
          }

          if (typeof options.always == 'function') {
            options.always(response);
          }
        });
      }
    };
  })

  //消息
  .factory('ImService', function ($interval, $rootScope, UserService, $ionicLoading, xc, http) {

    //聊天对象
    var chat = function () {
      this.pullingText = '下拉加载历史';
      this.toName = null;
      this.text = null;
      this.timer = null;
      this.listener = null;
      this.creatorId = 0;
      this.receiverId = 0;
      this.groupId = 0;
      this.creator = null;
      this.receiver = null;
      this.content = false;
      this.list = [];
      this.params = {
        isHasNext: true,
        isLastPage: false,
        pageNo: 1,
        pageSize: 10
      };
    };

    //设置已读
    chat.prototype.reset = function () {
      this.pullingText = '下拉加载历史';
      this.text = null;
      this.list = [];
      this.params = {
        isHasNext: true,
        isLastPage: false,
        pageNo: 1,
        pageSize: 10
      };
    };

    //获取基础信息
    chat.prototype.getGroupInfo = function (callback) {
      var that = this;
      http.get('im/getImGroupInfo', {
        aid: that.creatorId,
        receiver: that.receiverId,
        groupId: that.groupId
      }, function (response) {
        if (response.code == 0) {
          that.creator = response.data.creator;
          that.receiver = response.data.receiver;
          that.groupId = response.data.groupId;
          that.toName = response.data.receiver.name || response.data.receiver.telNumber;
          that.setReaded();
          if (typeof callback === 'function') callback();
        } else {
          $ionicLoading.show({
            template: '<div><i class="icon icon-2x ion-sad-outline"></i></div>' + response.msg,
            noBackdrop: true,
            hideOnStateChange: true,
            duration: 3000
          });
        }
      });
    };

    //设置已读
    chat.prototype.setReaded = function () {
      http.get('im/read', {
        aid: this.creatorId,
        groupId: this.groupId
      });
    };

    //加载历史消息
    chat.prototype.getHistory = function (callback) {

      var that = this;

      var pars = {
        aid: that.creatorId,
        groupId: that.groupId,
        receiver: that.receiverId,
        pageSize: that.params.pageSize,
        pageNo: that.params.pageNo
      };

      http.get('im/get', pars, function (response) {

        var data = response.data;

        that.params.isHasNext = data.isHasNextPage;

        that.params.isLastPage = data.isLastPage;

        that.params.pageNo = data.pageNum;

        if (that.params.pageNo == 1) {
          that.list = [];
        }

        for (var i = 0, len = data.list.length; i < len; i++) {
          var d = data.list[i];
          that.list.splice(0, 0, d);
        }

        if (typeof callback === 'function') callback(data);
      });
    };

    //发送新消息
    chat.prototype.send = function (callback) {

      var that = this;

      console.log('消息内容:' + chat.text);

      if (that.text !== null && that.text.length > 0) {

        var text = that.text;

        var newMsg = {
          avatar: that.creator.avatar,
          name: that.creator.name,
          userId: that.creator.id,
          content: text + '',
          createTime: new Date().getTime(),
          status: 0,
          ismine: true
        };

        that.list.push(newMsg);

        http.post('im/post', {
            aid: that.creatorId,
            groupId: that.groupId,
            receiver: that.receiverId
          }, {content: encodeURI(encodeURIComponent(text))},
          function () {
            that.text = null;
            newMsg.status = 1;
            if (typeof callback === 'function') callback();
          }, function () {
            that.text = null;
            newMsg.status = 2;
            if (typeof callback === 'function') callback();
          });
      }
    };

    //启动消息监听
    chat.prototype.startListener = function (callback) {
      var that = this;
      that.listener = $interval(function () {
        http.get('im/getNew', {
          aid: that.creatorId,
          groupId: that.groupId
        }, function (response) {
          if (response.code == 0 && response.data) {
            var len = response.data.length;
            if (len > 0) {
              for (var i = 0; i < len; i++) {
                that.list.push(response.data[i]);
              }
              if (typeof callback === 'function') callback();
            }
          }
        }, function () {
        });
      }, xc.app.interval);
    };

    //关闭消息监听
    chat.prototype.stopListener = function () {
      $interval.cancel(this.listener);
      delete this.listener;
    };

    //检查新消息
    chat.prototype.checkNews = function () {
      http.get('im/getUnReadCount', {
        aid: UserService.user.id
      }, function (response) {
        console.log('有未读消息' + response.data);
        $rootScope.unReadMsgCount = response.data;
      }, function () {
      });
    };

    //启动检查服务
    chat.prototype.startServer = function () {
      console.log('启动im消息监听服务');
      var that = this;
      that.timer = $interval(function () {
        that.checkNews();
      }, xc.app.interval);
    };

    //关闭检查服务
    chat.prototype.stopServer = function () {
      console.log('关闭im消息监听服务');
      $interval.cancel(this.timer);
      delete this.timer;
    };

    return new chat();
  })

  //教学管理资讯
  .factory('EducationArticleService', function (http, $ionicModal, $ionicPopup, $ionicLoading, xc) {

    return {
      request: function (options) {

        options.params.pageNum = options.params.pageNo;
        options.params.pageSize = xc.app.pageSize;

        return http.get('getDducationArticeList', options.params, function (response) {

          var data = response.data;

          options.params.isHasNext = data.isHasNextPage;

          options.params.isLastPage = data.isLastPage;

          options.params.pageNo = data.pageNum;

          if (options.params.pageNo == 1) {
            options.list = [];
          }

          for (var i = 0, len = data.list.length; i < len; i++) {
            var d = data.list[i];
            d.summary = d.summary.trim();
            options.list.push(d);
          }

          if (typeof options.always == 'function') {
            options.always(response);
          }
        });
      }
    };
  })

  //通知
  .factory('NoticeListService', function (http, $ionicLoading, xc) {

    return {
      request: function (options) {

        options.params.pageNum = options.params.pageNo;

        options.params.pageSize = xc.app.pageSize;

        return http.get('getAllNotice', options.params, function (response) {

          var data = response.data;

          options.params.isHasNext = data.isHasNextPage;

          options.params.isLastPage = data.isLastPage;

          options.params.pageNo = data.pageNum;

          if (options.params.pageNo == 1) {
            options.list = [];
          }

          for (var i = 0, len = data.list.length; i < len; i++) {
            var d = data.list[i];
            options.list.push(d);
          }

          if (typeof options.always == 'function') {
            options.always(response);
          }
        });
      }
    };
  })

  //作业
  .factory('HomeworkListService', function (http, xc) {

    var subjectColors = xc.app.subjectColors;

    return {
      request: function (options) {

        options.params.pageNum = options.params.pageNo;
        options.params.pageSize = xc.app.pageSize;

        return http.get('getAllHomework', options.params, function (response) {

          var data = response.data;

          options.params.isHasNext = data.isHasNextPage;

          options.params.isLastPage = data.isLastPage;

          options.params.pageNo = data.pageNum;

          if (options.params.pageNo == 1) {
            options.list = [];
          }

          for (var i = 0, len = data.list.length; i < len; i++) {
            var d = data.list[i];
            d.color = 'stable';
            for (var key in subjectColors) {
              if (d.subject == key) {
                d.color = subjectColors[key];
                break;
              }
            }
            options.list.push(d);
          }

          if (typeof options.always == 'function') {
            options.always(response.data);
          }
        });
      }
    };
  })

  //发布通知
  .factory('NoticeService', function (http, $rootScope, $ionicModal, $ionicLoading, $ionicPopup, xc, UserService) {

    var currentModal;

    return {
      close: function () {
        if (currentModal) {
          currentModal.hide();
        }
        currentModal = null;
      },
      open: function ($scope) {

        $ionicModal.fromTemplateUrl('templates/notice.html', {
          scope: $scope
        }).then(function (modal) {

            currentModal = modal;

            currentModal.show();

            var notice = {title: '', content: ''};

            $scope.notice = notice;

            var myClass = {list: []};

            $scope.class = myClass;

            http.get('myclass', {aid: UserService.user.id}, function (response) {
              myClass.list = response.data;
            });

            //提交按钮事件
            $scope.send = function () {

              var ids = [];

              for (var i = 0, len = myClass.list.length; i < len; i++) {
                if (myClass.list[i].checked) {
                  ids.push(myClass.list[i].classid);
                }
              }

              if (notice.title.length == 0) {
                $ionicPopup.alert({
                  title: '发布通知',
                  template: '请输入标题',
                  cancelText: '取消',
                  okText: '确认'
                });
                return;
              }

              if (notice.content.length == 0) {
                $ionicPopup.alert({
                  title: '发布通知',
                  template: '请输入内容',
                  cancelText: '取消',
                  okText: '确认'
                });
                return;
              }

              if (ids.length == 0) {
                $ionicPopup.alert({
                  title: '发布通知',
                  template: '请选择要发送到的班级',
                  cancelText: '取消',
                  okText: '确认'
                });
                return;
              }

              notice.classids = ids.join(',');

              http.get('publishNotice', {
                aid: UserService.user.id,
                title: encodeURI(encodeURIComponent((notice.title))),
                content: encodeURI(encodeURIComponent((notice.content))),
                classids: notice.classids
              }, function () {
                $scope.doRefresh();
                currentModal.hide();
              });

              $ionicLoading.show({
                template: '通知发布成功!',
                animation: 'fade-in',
                scope: $rootScope,
                showBackdrop: false,
                hideOnStateChange: false,
                duration: 1500,
                showDelay: 0
              });
            };
          }
        );
      },
      request: function (options) {

        options.params.pageNum = options.params.pageNo;

        return http.get('getAllNotice', options.params, function (response) {

          var data = response.data;

          options.params.isHasNext = data.isHasNextPage;

          options.params.isLastPage = data.isLastPage;

          options.params.pageNo = data.pageNum;

          if (options.params.pageNo == 1) {
            options.list = [];
          }

          for (var i = 0, len = data.list.length; i < len; i++) {
            options.list.push(data.list[i]);
          }

          if (typeof options.always == 'function') {
            options.always(response.data);
          }
        });
      },
      delete: function (id) {
        return http.get('deleteNotice', {
          aid: UserService.user.id,
          nid: id
        });
      }
    };
  })

  //发布作业
  .factory('HomeworkService', function (http, $ionicModal, $ionicLoading, $ionicPopup, xc, UserService) {

    var currentModal;

    return {
      close: function () {
        if (currentModal) {
          currentModal.hide();
        }
        currentModal = null;
      },
      open: function ($scope) {

        $ionicModal.fromTemplateUrl('templates/homework.html', {
          scope: $scope
        }).then(function (modal) {

            currentModal = modal;

            currentModal.show();

            var homework = {
              content: '',
              classList: {},
              subjectList: {},
              classId: '',
              subjectId: ''
            };

            $scope.homework = homework;

            http.get('myclass', {aid: UserService.user.id}, function (response) {
              homework.classList.list = response.data;
            });

            $scope.classChange = function (cid) {
              http.get('getSubject', {classid: cid}, function (response) {
                homework.subjectList.list = response.data;
              });
            };

            //提交按钮事件
            $scope.send = function () {

              if (homework.classId.length == 0) {
                $ionicPopup.alert({
                  title: '发布作业',
                  template: '请选择班级',
                  cancelText: '取消',
                  okText: '确认'
                });
                return;
              }

              if (homework.subjectId.length == 0) {
                $ionicPopup.alert({
                  title: '发布作业',
                  template: '请选择科目',
                  cancelText: '取消',
                  okText: '确认'
                });
                return;
              }

              if (homework.content.length == 0) {
                $ionicPopup.alert({
                  title: '发布作业',
                  template: '请输入内容',
                  cancelText: '取消',
                  okText: '确认'
                });
                return;
              }

              var data = {
                aid: UserService.user.id,
                content: encodeURI(encodeURIComponent((homework.content))),
                classids: homework.classId,
                subject: homework.subjectId
              };

              http.get('publishHomework', data, function (res) {
                $ionicLoading.show({
                  template: '作业布置成功!',
                  animation: 'fade-in',
                  showBackdrop: false,
                  hideOnStateChange: false,
                  scope: $scope,
                  duration: 1000,
                  showDelay: 100
                });
                $scope.doRefresh();
                currentModal.hide();
              });
            };
          }
        );
      },
      request: function (options) {

        options.params.pageNum = options.params.pageNo;

        return http.get('getAllNotice', options.params, function (response) {

          var data = response.data;

          options.params.isHasNext = data.isHasNextPage;

          options.params.isLastPage = data.isLastPage;

          options.params.pageNo = data.pageNum;

          if (options.params.pageNo == 1) {
            options.list = [];
          }

          for (var i = 0, len = data.list.length; i < len; i++) {
            var d = data.list[i];
            options.list.push(d);
          }

          if (typeof options.always == 'function') {
            options.always(response.data);
          }
        });
      },
      delete: function (id) {
        return http.get('deleteHomework', {
          aid: UserService.user.id,
          hid: id
        });
      }
    };
  })

  //课程分类
  .factory('ClassZoneService', function (http, $stateParams, $ionicLoading, xc, UserService) {

    var myClass = {error: null, list: null, total: 0};

    return {
      class: myClass,
      load: function () {
        http.get('myclassZone', {aid: UserService.user.id}, function (response) {
          myClass.list = response.data;
          myClass.total = 0;
          for (var i = 0, len = myClass.list.length; i < len; i++) {
            myClass.total += myClass.list[i].picturecount;
          }
        });
      }
    };
  })

  //相册
  .factory('GalleryService', function (http) {
    return {
      request: function (options) {

        options.params.pageNum = options.params.pageNo;

        return http.get('myclassAlbums', options.params, function (response) {

          var data = response.data;

          options.params.isHasNext = data.isHasNextPage;

          options.params.isLastPage = data.isLastPage;

          options.params.pageNo = data.pageNum;

          if (options.params.pageNo == 1) {
            options.list = [];
          }

          for (var i = 0, len = data.list.length; i < len; i++) {
            var d = data.list[i];
            options.list.push(d);
          }

          if (typeof options.always == 'function') {
            options.always(response.data);
          }
        });
      }
    };
  })

  //图片列表
  .factory('PicService', function (http, xc) {
    return {
      getList: function (options) {
        return http.get('myClassPictures', options.params,
          function (response) {

            var data = response.data;

            if (data) {

              options.params.isHasNext = data.isHasNextPage;

              options.params.isLastPage = data.isLastPage;

              options.params.pageNo = data.pageNum;

              for (var i = 0, len = data.list.length; i < len; i++) {
                data.list[i].thumb = xc.server.cdn + data.list[i].thumb;
                data.list[i].src = xc.server.cdn + data.list[i].original;
                options.items.push(data.list[i]);
              }

              if (typeof options.success == 'function') {
                options.success(response.data);
              }
            }

            if (typeof options.always == 'function') {
              options.always(response.data);
            }
          });
      },
      delete: function (item) {
        http.get('deleteClassAlbum', {albumId: item.id});
      }
    };
  })

  //上传图片
  .factory('UploadService', function (http, $stateParams, $location, $ionicLoading, $cordovaImagePicker, xc, UserService) {

    var uploadParms = [];
    var classId;
    var callback;
    return {
      choose: function (_classId, _callback) {
        classId = _classId;
        callback = _callback;
        var options = {
          maximumImagesCount: 10,
          width: 800,
          height: 800,
          quality: 80
        };
        var that = this;
        $cordovaImagePicker.getPictures(options)
          .then(function (results) {
            for (var i = 0, len = results.length; i < len; i++) {
              console.log('上传第一张:' + results[i]);
              $ionicLoading.show({
                template: '<i class="ion-load-a"></i>正在上传第' + (i + 1) + '张,共' + len + '张',
                animation: 'fade-in',
                showDelay: 0
              });
              that.upload(results[i], i == len - 1);
            }
          });
      },
      upload: function (imageURI, islast) {

        var that = this;

        console.log('上传图片');

        var options = new FileUploadOptions();

        //用于设置参数，对应form表单里控件的name属性，这是关键，废了一天时间，完全是因为这里，这里的参数名字，和表单提交的form对应
        options.fileKey = "file";

        options.fileName = imageURI.substr(imageURI.lastIndexOf('/') + 1);

        //如果是图片格式，就用image/jpeg，其他文件格式上官网查API
        options.mimeType = "image/jpeg";

        options.chunkedMode = false;

        //options.mimeType = "multipart/form-data";//这两个参数修改了，后台就跟普通表单页面post上传一样 enctype="multipart/form-data"

        //这里的uri根据自己的需求设定，是一个接收上传图片的地址
        var uri = encodeURI(xc.server.base + 'imageUpload');

        var ft = new FileTransfer();

        ft.upload(imageURI, uri, function (response) {

          var d = JSON.parse(response.response).data;

          uploadParms.push(d);

          if (islast) {
            //保存
            console.log('保存到业务');
            that.save();
          }

        }, function () {
        }, options);
      },
      save: function () {
        http.get('saveClassUpload', {
          classId: classId,
          userId: UserService.user.id,
          image: JSON.stringify(uploadParms)
        }, function (response) {
          if (typeof callback == 'function')
            callback();
          uploadParms = [];
        });
      }
    }
  })

  //课程分类
  .factory('CourseCategoryService', function (http) {

    var category = {error: null, categories: null};

    return {
      category: category,
      load: function () {
        http.get('getTutorKnowledgeCategory', {}, function (response) {
          category.categories = response.data;
        });
      }
    };
  })

  //课程分类
  .factory('EducationCategoryService', function (http) {

    var category = {error: null, categories: null};

    return {
      category: category,
      load: function () {
        http.get('getEducationCategory', {}, function (response) {
          category.categories = response.data;
        });
      }
    };
  })

  //专栏
  .factory('SpecialCategoryService', function (http) {

    var category = {error: null, categories: null};

    return {
      category: category,
      load: function () {
        http.get('getCategory', {
          module: 'special',
          pointId: 0
        }, function (response) {
          var result = response.data;
          category.categories = [];
          for (var i = 0, len = result.length; i < len; i++) {
            category.categories.push(result[i]);
          }
        });
      }
    };
  });
